/* DC-Motor PWM Control Program
 Program By GucciTech - Chojiro Higashiguchi
 Date 2009/04/26 Ver.2
 Program Name 'DC-M_kakuhan.C' Version2

 I/O Port PIN
   Vdd(1Pin) (2V-5.5V)
   Vss(8Pin)
   GP0(7Pin)  Monitor LED
   GP1(6Pin)  Gate_out
   GP2(5Pin)  PWM_out
   GP3(4Pin)  Cycle_SW
   GP4(3Pin)  Speed_SW
   GP5(2Pin)  Continu_SW
*/

#include "J:\GucciTech\C Study\CCS-C PIC\12F683\DC-Motor_Control1\V2\DC-M_kakuhan.h"

void t2_start(void);
void t2_stop(void);

static unsigned long duty;
static unsigned int SW_state;
static unsigned int SW_buffer;

#define ON_time 1000
#define OFF_time 1500
#define T2CON 0x12
#define TMR2ON 2

// TIMER0 Interruput Routine
#int_TIMER0
void  TIMER0_isr(void) 
{
   set_timer0(256-32);           //(4/8MHz)*256*64=8.192msec
   if(input(Speed_SW) == 0)
   {
      if(++duty >= 1000)
      {
         duty = 100;
         set_pwm1_duty(duty);  // Duty 0% Set
      }
      else
         set_pwm1_duty(duty);
   }
}


void t2_start(void)
{
   #asm
      bsf   T2CON,TMR2ON
   #endasm
}

void t2_stop(void)
{
   #asm
      bcf   T2CON,TMR2ON
   #endasm

}

void main()
{
   setup_oscillator(OSC_8MHZ|OSC_INTRC);
   setup_adc_ports(NO_ANALOGS|VSS_VDD);
   setup_adc(ADC_OFF);
   set_tris_a(0x38);             // GP0-2='OUT',GP3-5='IN'
   port_a_pullups(0b00110000);   // setup_timer_0(RTCC_INTERNAL|RTCC_DIV_1);
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_256);      //Prescaler 1/256 InsideClock
   set_timer0(256-32);                             //(4/8MHz)*256*32=4.096msec
   setup_timer_1(T1_DISABLED);
   //setup_timer_2(T2_DIV_BY_4,249,1);             // Period=500uS(2kHz)
   setup_timer_2(T2_DIV_BY_16,249,1);              // Period=2mS(500Hz)
   t2_stop();                    // TMR2 'STOP'
   setup_ccp1(CCP_PWM);
   setup_comparator(NC_NC);
   setup_vref(FALSE);

   // TODO: USER CODE!!
   output_low(Monitor);
   output_low(Gate_out);
   output_low(PWM_out);
   set_pwm1_duty(0);          // Duty 20% Set

   output_high(Monitor);
   delay_ms(1000);            // 1sec Wait
   output_low(Monitor);
   
   duty = 100;

/// Interrupts Enable Set
   enable_interrupts(INT_TIMER0);
   enable_interrupts(GLOBAL);

   while(true)
   {
      SW_state = input_a() & 0b00101000;
      SW_buffer = SW_state;
      // STOP ------------------------------------
      if(SW_state == 0x28)
      {
         t2_stop();                 // TMR2 'STOP'
         set_pwm1_duty(0);          // Duty 0% Set
         output_low(Gate_out);
         output_low(PWM_out);
         output_low(Monitor);
         while(SW_buffer == SW_State){
            SW_state = input_a() & 0b00101000;
         }
      }
      // Continu-Mode -----------------------------
      if(input(Continu_SW) == 0)
      {
         t2_start();                // TMR2 'START'
         set_pwm1_duty(duty);       // Duty Set
         output_high(Gate_out);
         output_high(PWM_out);
         output_high(Monitor);
         while(input(Continu_SW) == 0){
         }
      }
      // Cycle-Mode -------------------------------
      if(input(Cycle_SW) == 0)
      {
         while(input(Cycle_SW) == 0)
         {
            t2_start();                // TMR2 'START'
            set_pwm1_duty(duty);       // Duty Set
            output_high(Gate_out);
            output_high(PWM_out);
            output_high(Monitor);
            delay_ms(ON_time);
            t2_stop();                 // TMR2 'STOP'
            set_pwm1_duty(0);          // Duty 0% Set
            output_low(Gate_out);
            output_low(PWM_out);
            output_low(Monitor);
            delay_ms(OFF_time);
         }
      }
   }
}
